<?php

namespace Drupal\org_chart_block\Plugin\Block;

use Drupal\Core\Block\BlockBase;
use Drupal\Core\Cache\Cache;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a 'OrganizationBlock' block.
 *
 * @Block(
 *  id = "organization_block",
 *  admin_label = @Translation("Organization block"),
 * )
 */
class OrganizationBlock extends BlockBase implements ContainerFactoryPluginInterface {


  const ORGANIZATION_TERM = 'organization_chart_block';
  
  /**
   * Drupal\Core\Entity\EntityTypeManagerInterface definition.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $instance = new static($configuration, $plugin_id, $plugin_definition);
    $instance->entityTypeManager = $container->get('entity_type.manager');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
    ] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function blockForm($form, FormStateInterface $form_state) {
    $options = [];

    /** @var \Drupal\taxonomy\TermInterface  $organization_terms */
    $organization_terms = $this->entityTypeManager->getStorage("taxonomy_term")->loadByProperties(['vid' => self::ORGANIZATION_TERM]);
    foreach ($organization_terms as $organization_term) {
      $options[$organization_term->id()]= $organization_term->getName();
    };

    $form['root'] = [
      '#type' => 'select',
      '#title' => $this->t('Root'),
      '#default_value' => $this->configuration['root'],
      '#options' => $options,
      '#weight' => '0',
    ];
    $form['background_color'] = [
      '#type' => 'color',
      '#title' => $this->t('Background Color'),
      '#default_value' =>  $this->configuration['background_color'],
      '#weight' => '0',
    ];
    $form['color'] = [
      '#type' => 'color',
      '#title' => $this->t('Color'),
      '#default_value' =>  $this->configuration['color'],
      '#weight' => '0',
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state) {
    $this->configuration['root'] = $form_state->getValue('root');
    $this->configuration['color'] = $form_state->getValue('color');
    $this->configuration['background_color'] = $form_state->getValue('background_color');
  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    $organization_tree = $this->tree_data();
    $build = [];
    $build['organization_block'] = [
      '#theme' => 'organization_chart_block',
      '#attached' => [
        'library' => [
          'org_chart_block/organization-chart'
        ],
        'drupalSettings' => [
          'organization' => [
            'background_color' => $this->configuration['background_color'],
            'color' => $this->configuration['color'],
            'organization_tree' => $organization_tree
          ]
        ]
      ]
    ];
    return $build;
  }

  /**
   * {@inheritdoc}
   */
  private function tree_data() {
    $tree = [];
    $organizations_tree = $this->entityTypeManager->getStorage('taxonomy_term')->loadTree(self::ORGANIZATION_TERM, $this->configuration['root']);
    /** @var \Drupal\taxonomy\TermInterface  $root */
    $root = $this->entityTypeManager->getStorage('taxonomy_term')->load($this->configuration['root']);
    if( $root ) {

      $tree_data = [
        [
          'v' => $root->getName(),
          'f' => "{$root->getName()},{$root->get('field_organization_email')->value},{$root->get('field_organization_position')->value}"
         ],
        '',
      ];

      $tree[] = $tree_data;
    }
  
    foreach ( $organizations_tree as  $organization_tree) {
      /** @var \Drupal\taxonomy\TermInterface  $term */
      $term = $this->entityTypeManager->getStorage('taxonomy_term')->load($organization_tree->tid);
      /** @var \Drupal\taxonomy\TermInterface  $term_parent */
      $term_parent = $this->entityTypeManager->getStorage('taxonomy_term')->load($term->get('parent')->target_id); 

      $tree_data = [
         
        [
         'v' => $term->getName(),
         'f' => "{$term->getName()},{$term->get('field_organization_email')->value},{$term->get('field_organization_position')->value}"
        ],
        $term_parent->getName(),
      ];
      $tree[] = $tree_data;
    };

    return $tree;
  }
  
  public function getCacheTags() {
    // With this when your parent change your block will rebuild
    return Cache::mergeTags(parent::getCacheTags(), array('term:' .  $this->entityTypeManager->getStorage('taxonomy_term')->load($this->configuration['root'])->id()));
  }

  public function getCacheContexts() {
    // Every new route this block will rebuild
    return Cache::mergeContexts(parent::getCacheContexts(), array('route'));
  }
}
